import logging
from pathlib import Path


class ErrorHandler:

    def __init__(self, className):
        self.errorlog = []

        self.logger = logging.getLogger(className)

        c_handler = logging.StreamHandler()
        # f_handler = logging.StreamHandler()
        logpath = './thundera.log'
        filename = Path(logpath)
        filename.touch(exist_ok=True)
        f_handler = logging.FileHandler(logpath)
        c_handler.setLevel(logging.INFO)
        f_handler.setLevel(logging.ERROR)

        # Create formatters and add it to handlers
        c_format = logging.Formatter('%(name)s - %(levelname)s - %(message)s')
        f_format = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        c_handler.setFormatter(c_format)
        f_handler.setFormatter(f_format)

        # Add handlers to the logger
        self.logger.addHandler(c_handler)
        self.logger.addHandler(f_handler)

    def error(self, message):
        self.logger.error(message)
        # print(message)

    def info(self, message):
        self.logger.info(message)
        # print(message)



import os
import re
import json
import mimetypes
import hashlib
import magic
import subprocess
from slugify import slugify
from binaryornot.check import is_binary
from . import FileHandler
from . import ExifHandler


class File(object):
    def __init__(self, errorHandler, filepath, filename, pfile):
        self.debug = errorHandler
        self.filepath = filepath
        self.filename = filename
        targetF = os.path.join(filepath, filename)
        self.checksum = self.file_checksum(targetF)
        mime = magic.Magic(mime=True)
        self.filetype = mime.from_file(targetF)
        self.isarchive = self.is_archive(self.filetype)
        self.metadata = self.metadata(
            self.filetype, os.path.join(filepath, filename))
        self.filesize = self.file_size(targetF)
        self.slugify = slugify(filename)
        self.pfile = pfile
        self.isbinary = is_binary(os.path.join(filepath, filename))
        self.forceign = self.check_ignore(self.filetype, filepath, filename)
        self.plang = 'none'
        self.symstat = 0
        self.csv = {}
        if not self.isarchive and not self.forceign:
            fCSV = os.path.splitext(filename)[0]
            fileHandler = FileHandler.FileHandler(
                self.debug,
                filepath,
                filename,
                self.filetype,
                self.checksum)
            self.csv[fCSV] = fileHandler.run_handler()

    def get_symbols(self):
        rCSV = {}
        for key, value in self.csv.items():
            if value:
                rCSV[key] = value
                self.symstat = len(value)
        return rCSV

    @classmethod
    def new_to_dict(cls, filepath, filename):
        self.checksum = self.file_checksum(os.path.join(filepath, filename))
        dictionary[checksum] = cls(filepath, filename)

    def convert_bytes(self, num):
        for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
            if num < 1024.0:
                return "%3.1f %s" % (num, x)
            num /= 1024.0

    def check_ignore(self, file_type, filepath, filename):
        list_mimes = [
            'application/x-git',
            'text/plain'
            ]
        list_subfolders = [
            '.git'
            ]
        if file_type in list_mimes:
            return True
        else:
            for sf in list_subfolders:
                if sf in filepath:
                    return True
                else:
                    return False

    def is_archive(self, file_type):
        # This function needs to be replaced by the info from ArchiveHandler
        list_mimes = [
            'application/java-archive',
            'application/zip',
            'application/java-archive',
            'application/gzip',
            'application/zlib',
            'application/x-tar'
            ]
        if file_type in list_mimes:
            return True
        else:
            return False

    def metadata(self, file_type, file_path):
        # This function needs to be replaced by the info from ArchiveHandler
        list_mimes = [
            'application/x-dosexec',
            'application/x-sharedlib',
            'font/sfnt'
            ]
        if file_type in list_mimes:
            exif = ExifHandler.ExifHandler(file_path)
            return exif.run()
        else:
            return ''

    def file_size(self, file_path):
        if os.path.isfile(file_path):
            file_info = os.stat(file_path)
            return self.convert_bytes(file_info.st_size)

    def file_checksum(self, file_path):
        with open(file_path, "rb") as f:
            file_hash = hashlib.md5()
            chunk = f.read(8192)
            while chunk:
                file_hash.update(chunk)
                chunk = f.read(8192)
        return file_hash.hexdigest()



import os
import time
import mimetypes
import hashlib
import tarfile
import gzip
import csv
import sys
from slugify import slugify
from zipfile import ZipFile
from tabulate import tabulate
from . import File
from . import ArchiveHandler
from . import ErrorHandler


class FolderScanner:

    def __init__(self, rootfolder):

        self.debug = ErrorHandler.ErrorHandler(__name__)
        ts = time.time()

        # Files found during scanner
        self.filelist = []

        # Files to Exclude from ArchiveHandler
        self.exfiles = {}

        # List of folders
        self.folders = {}
        self.symbols = {}

        self.rootfolder = rootfolder
        self.scanFolder(rootfolder, '-')
        self.rfname = slugify(rootfolder)

        self.fresults = []
        self.aresults = []
        self.bsaqueue = []

        self.file_count
        self.lcounter = 0

        for fO in self.filelist:
            # print (fO.filename)
            row = [
                fO.checksum,
                fO.filename,
                fO.filetype,
                fO.isbinary,
                fO.isarchive,
                fO.forceign,
                fO.plang,
                fO.pfile,
                fO.filepath]
            if fO.isbinary and not fO.isarchive and not fO.forceign:
                self.bsaqueue.append(row)
            elif fO.isarchive and not fO.forceign:
                self.aresults.append(row)
            else:
                if not fO.forceign:
                    self.fresults.append(row)

        headers = [
            "Checksum",
            "Filename",
            "MimeType",
            "Binary",
            "Archive",
            "Ignore",
            "Language",
            "Parent",
            "Filepath"]

        with open(self.rfname+'-archive.csv', 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(headers)
            for row in self.aresults:
                writer.writerow(row)

        with open(self.rfname+'-files.csv', 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(headers)
            for row in self.fresults:
                writer.writerow(row)

        headers = [
            "Checksum",
            "Symbols"]

        rowsym = []
        for key in self.symbols:
            rowsym.append([key, self.symbols[key]])

        with open(self.rfname+'-symbols.csv', 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(headers)
            for row in rowsym:
                writer.writerow(row)

        # print(self.symbols)
        # for key in self.symbols:
        #    print(key, '->', self.symbols[key])

        dt = str(round(time.time() - ts, 2))
        self.debug.info('Scan took ' + dt + ' seconds')

    def progress(self, count, total):
        bar_len = 60
        filled_len = int(round(bar_len * count / float(total)))

        percents = round(100.0 * count / float(total), 1)
        bar = '=' * filled_len + '-' * (bar_len - filled_len)

        sys.stdout.write(
            '[%s] %s%s %s\r' % (bar, percents, '%', 'scanning...'))
        sys.stdout.flush()

    def scanFolder(self, rootfolder, pfile):
        if len(pfile.strip()) <= 1:
            eMSG = "* Preparing files to be scanned, please wait..."
            print(eMSG)
            kount = [len(files) for _, _, files in os.walk(rootfolder)]
            self.file_count = kount
            self.file_count = sum(self.file_count)
            print(' ', '>', self.file_count, 'files added to the queue.')
            self.lcounter = 0

        for (dirpath, dirnames, filenames) in os.walk(rootfolder):
            for f in filenames:
                if len(pfile.strip()) <= 1:
                    self.lcounter += 1
                    self.progress(self.lcounter, self.file_count)
                fileInfo = File.File(self.debug, dirpath, f, pfile)
                self.filelist.append(fileInfo)
                symbols = fileInfo.get_symbols()
                if len(symbols) >= 1:
                    tmpCSV = symbols[list(symbols.keys())[0]].split(',', 1)
                    self.symbols[tmpCSV[0]] = tmpCSV[1]
            for d in dirnames:
                self.folders[d] = dirpath
        if len(pfile.strip()) <= 1:
            sys.stdout.flush()
            print('')
        for ar in self.filelist:
            if ar.isarchive:
                print(' ', '>', ar.filename)
                archfile = os.path.join(ar.filepath, ar.filename)
                if archfile not in self.exfiles.keys():
                    target = os.path.join(ar.filepath, ar.slugify)
                    self.exfiles[archfile] = target
                    ArchiveHandler.ArchiveHandler(
                        self.debug,
                        ar.filepath,
                        ar.filename,
                        ar.filetype,
                        ar.slugify,
                        ar.checksum
                    )
                    self.scanFolder(target, ar.checksum)
